/**
 * ps
 *
 * Tool to print process information for all processes running in your
 * system. Outputs results in a format similar to UNIX ps command.
 *
 * Works with Windows 95/98 and Windows 2000.
 * 
 * Usage:
 *     ps
 *
 * File:    ps.c
 * Author:  Tanmay K. Mohapatra
 * E-Mail:  tanmaykm@netscape.net
 * HTTP:    http://members.tripod.com/~tanmaykm/
 *          http://tanmaykm.webjump.com/
 *
 */

#include <windows.h>
#include <tlhelp32.h>
#include <stdio.h>

static void printErr(const char *pErrStage, HRESULT hErrResult);
static int  printSnapShot(HANDLE pSnapShot);

int main(int argc, char **argv)
{
    HANDLE      pSnapShot;

    pSnapShot = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS | TH32CS_SNAPTHREAD, 0);

    if((HANDLE)-1 == pSnapShot)
    {
        printErr("Error getting snapshot", GetLastError());
        return -1;
    }

    if (0 != printSnapShot(pSnapShot))
    {
        return -1;
    }

    CloseHandle(pSnapShot);
    return 0;
}

static int printSnapShot(HANDLE pSnapShot)
{
    PROCESSENTRY32  stProcEntry;
    BOOL            bRetCode;
    DWORD           dwLastErr;

    printf("\nPID\tPPID\tTHRD\tUSG\tPRI\tPATH\n");
    for(stProcEntry.dwSize=sizeof(stProcEntry), bRetCode = Process32First(pSnapShot, &stProcEntry);
        bRetCode;
        stProcEntry.dwSize=sizeof(stProcEntry), bRetCode = Process32Next(pSnapShot, &stProcEntry))
    {
        printf("%u\t%u\t%u\t%u\t%u\t%s\n", 
            stProcEntry.th32ProcessID, 
            stProcEntry.th32ParentProcessID, 
            stProcEntry.cntThreads, 
            stProcEntry.cntUsage, 
            stProcEntry.pcPriClassBase,
            stProcEntry.szExeFile);
    }

    dwLastErr = GetLastError();
    if (ERROR_NO_MORE_FILES  != dwLastErr)
    {
        printErr("Error getting process information", dwLastErr);
        return -1;
    }
    return 0;
}

static void printErr(const char *pErrStage, HRESULT hErrResult)
{
    LPTSTR  pCompleteErrMsg;

    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
        NULL, hErrResult,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
        (LPTSTR)&pCompleteErrMsg,
        0, NULL);

    fprintf(stderr, "%s (Error %d): %s", pErrStage, hErrResult, pCompleteErrMsg);
    LocalFree (pCompleteErrMsg);
}
